@@ -383,10 +383,15 @@ Can be used to turn version control on or off."
   :group 'gnus-newsrc
   :type 'hook)
 
-;;; Internal variables
+(defcustom gnus-always-read-dribble-file nil
+  "Uncoditionally read the dribble file."
+  :group 'gnus-newsrc
+  :type 'boolean)
 
-(defvar gnus-always-read-dribble-file nil
-  "Uncoditionally read the dribble file.")
+(defvar gnus-startup-file-coding-system 'binary
+  "*Coding system for startup file.")
+
+;;; Internal variables
 
 (defvar gnus-newsrc-file-version nil)
 (defvar gnus-override-subscribe-method nil)
@@ -424,7 +429,9 @@ Can be used to turn version control on or off."
 		   (file-exists-p (concat file ".el"))
 		   (file-exists-p (concat file ".elc")))
 	       (condition-case var
-		   (load file nil t)
+		   (let ((coding-system-for-read
+			  gnus-startup-file-coding-system))
+		     (load file nil t))
 		 (error
 		  (error "Error in %s: %s" file var)))))))))
 
@@ -1484,7 +1491,7 @@ newsgroup."
 	  (when (<= (gnus-info-level info) foreign-level)
 	    (setq active (gnus-activate-group group 'scan))
 	    ;; Let the Gnus agent save the active file.
-	    (when (and gnus-agent gnus-plugged)
+	    (when (and gnus-agent gnus-plugged active)
 	      (gnus-agent-save-group-info
 	       method (gnus-group-real-name group) active))
 	    (unless (inline (gnus-virtual-group-p group))
@@ -1897,7 +1904,8 @@ If FORCE is non-nil, the .newsrc file is read."
     (gnus-message 5 "Reading %s..." ding-file)
     (let (gnus-newsrc-assoc)
       (condition-case nil
-	  (load ding-file t t t)
+	  (let ((coding-system-for-read gnus-startup-file-coding-system))
+	    (load ding-file t t t))
 	(error
 	 (ding)
 	 (unless (gnus-yes-or-no-p
